@@ -6,7 +6,7 @@ Mojolicious::Plugin::Bootstrap3 - Mojolicious + http://getbootstrap.com/
 
 =head1 VERSION
 
-3.2005
+3.3006
 
 =head1 DESCRIPTION
 
@@ -149,10 +149,20 @@ This is EXPERIMENTAL and subject to change.
 =cut
 
 use Mojo::Base 'Mojolicious::Plugin';
-use File::Spec::Functions 'catdir';
-use Cwd ();
+use Mojo::Util ();
+use File::Basename 'dirname';
+use File::Copy ();
+use File::Path ();
+use File::Spec ();
+use Cwd        ();
+use constant DEBUG => $ENV{MOJO_ASSETPACK_DEBUG} || 0;
 
-our $VERSION = '3.2005';
+our $VERSION = '3.3006';
+our $OVERRIDE;    # ugly hack. might go away
+
+$ENV{SASS_PATH} ||= '';
+
+my $ASSET_DIR = do { local $_ = Cwd::abs_path(__FILE__); s!\.pm$!!; $_; };
 
 my @DEFAULT_CSS_FILES = qw( bootstrap.scss );
 my @DEFAULT_JS_FILES
@@ -173,13 +183,14 @@ the C<SASS_PATH> environment variable.
 =cut
 
 sub asset_path {
-  my ($class, $type) = @_;
-  my $path = Cwd::abs_path(__FILE__);
+  my ($self, $type) = @_;
+  my @path = ref $self ? @{$self->{sass_path}} : ();
+  my %PATH;
 
-  $path =~ s!\.pm$!!;
-
-  return join ':', grep {$_} catdir($path, 'sass'), $ENV{SASS_PATH} if $type and $type eq 'sass';
-  return $path;
+  return join ':', grep { -d $_ and !$PATH{$_}++ } split(/:/, $ENV{SASS_PATH}), @path,
+    File::Spec->catdir($ASSET_DIR, 'sass')
+    if $type and $type eq 'sass';
+  return $ASSET_DIR;
 }
 
 =head2 register
@@ -187,8 +198,10 @@ sub asset_path {
   $app->plugin(
     bootstrap3 => {
       css => [qw( bootstrap.scss )],
+      custom => $bool, # default false
       js => [qw( button.js collapse.js ... )],
       jquery => $bool, # default true
+      theme => { paper => "https://bootswatch.com/paper/_variables.scss" }, # default to no theme
     },
   );
 
@@ -204,6 +217,11 @@ The name of the files to include in the asset named C<bootstrap.css>.
 
 Specify an empty list to disable building C<bootstrap.css>.
 
+=item * custom
+
+Disabled by default. Will copy C<sass/bootstrap.scss> to your project if
+true and set C<SASS_PATH> to the appropriate paths.
+
 =item * js
 
 C<affix.js>, C<alert.js>, C<button.js>, C<carousel.js>, C<collapse.js>,
@@ -222,6 +240,15 @@ in the current version.
 This will include the bundled L<jQuery|http://jquery.com/> version in the
 L<bootstrap.js> asset. Set this to 0 if you include your own jQuery.
 
+=item * theme
+
+Can be set to an name/URL hash, where the "name" is theme name, and the URL
+points to a custom C<_variables.scss> file. This file will be downloaded and
+put into your C<public/sass/bootstrap> directory, unless a C<_variables.scss>
+file already exists.
+
+Specifying a theme will override L</custom> and L</css>.
+
 =back
 
 =cut
@@ -231,15 +258,25 @@ sub register {
 
   $app->plugin('AssetPack') unless eval { $app->asset };
 
+  $self->{sass_path} = [];
   $config->{css} ||= [@DEFAULT_CSS_FILES];
   $config->{js}  ||= [@DEFAULT_JS_FILES];
+  $config->{custom} = 0 if $config->{theme};
   $config->{jquery} //= 1;
 
   push @{$app->static->paths}, $self->asset_path;
 
-  # TODO: 'bootstrap_resources.scss'
-  if (@{$config->{css}}) {
-    local $ENV{SASS_PATH} = $self->asset_path('sass');
+  if ($config->{custom}) {
+    $self->_copy_files($app,
+      map { [$_, $_] } ref $config->{custom} eq 'ARRAY' ? @{$config->{custom}} : @DEFAULT_CSS_FILES);
+  }
+
+  if ($config->{theme}) {
+    $self->_generate_theme($app, $config->{theme});
+  }
+  elsif (@{$config->{css}}) {
+    $ENV{SASS_PATH} = $self->asset_path('sass');
+    warn "[BOOTSTRAP] Defining asset 'bootstrap.css' SASS_PATH=$ENV{SASS_PATH}\n" if DEBUG;
     $app->asset('bootstrap.css' => map {"/sass/$_"} @{$config->{css}});
   }
 
@@ -251,6 +288,82 @@ sub register {
   }
 }
 
+sub _copy_files {
+  my $modifier = ref $_[-1] eq 'CODE' ? pop : sub { };
+  my ($self, $app, @files) = @_;
+
+  for (@files) {
+    my ($from, $to) = @$_;
+    my $source = File::Spec->catfile($ASSET_DIR, 'sass', split '/', $from);
+    my $destination = $self->_destination_file($app, $to) or next;
+    File::Path::make_path(dirname $destination) unless -d dirname($destination);
+    $app->log->info("[BOOTSTRAP] Copying $source to $destination");
+    local $_ = Mojo::Util::slurp($source);
+    $modifier->();
+    Mojo::Util::spurt($_, $destination);
+  }
+}
+
+sub _destination_file {
+  my ($self, $app, $name) = @_;
+  my $static = $app->static;
+
+  for my $path (@{$static->paths}) {
+    my $destination_dir = File::Spec->catdir($path, 'sass');
+    my $destination = File::Spec->catfile($destination_dir, split '/', $name);
+    push @{$self->{sass_path}}, $destination_dir;
+    return ''                                   if -e $destination;               # already exists
+    warn "[BOOTSTRAP] Can write $destination\n" if DEBUG;
+    return $destination                         if -w dirname $destination_dir;
+  }
+
+  # should never come to this, because of
+  # push @{$app->static->paths}, $self->asset_path;
+  $app->log->warn("Custom file $name does not exist in static directories!");
+  return '';
+}
+
+sub _generate_theme {
+  my ($self, $app, $theme) = @_;
+
+  for my $name (keys %$theme) {
+    my $url = $theme->{$name};
+
+    warn "[BOOTSTRAP] Defining theme '$name' from $url\n" if DEBUG;
+
+    if ($url =~ m!/_bootswatch\.scss!) {
+      my $destination = $self->_destination_file($app, "$name/_bootswatch.scss");
+      $self->_move($app->asset->fetch($url), $destination) if $destination and !-e $destination;
+      $url =~ s!/_bootswatch\.scss!/_variables.scss!;
+      local $OVERRIDE = 'bootswatch';
+      $self->_generate_theme($app, {$name => $url});
+    }
+    else {
+      my $destination = $self->_destination_file($app, "$name/_variables.scss");
+      if ($destination and !-e $destination) {
+        $self->_copy_files(
+          $app,
+          ["bootstrap.scss" => "$name.scss"],
+          sub {
+            s!(\@import.*bootstrap/variables.*)!\@import "$name/variables";\n$1!m;
+            s!(//.*\bUtility\b.*)!$1\n\@import "$name/$OVERRIDE";\n!mi if $OVERRIDE;
+          }
+        );
+        $self->_move($app->asset->fetch($url), $destination);
+      }
+      $ENV{SASS_PATH} = $self->asset_path('sass');
+      warn "[BOOTSTRAP] Defining asset '$name.css' SASS_PATH=$ENV{SASS_PATH}\n" if DEBUG;
+      $app->asset("$name.css" => "/sass/$name.scss");
+    }
+  }
+}
+
+sub _move {
+  my ($self, $from, $to) = @_;
+  File::Path::make_path(dirname $to);
+  File::Copy::move($from, $to) or die "[BOOTSTRAP] move $from $to: $!";
+}
+
 =head1 CREDITS
 
 L<bootstrap-sass|https://github.com/twbs/bootstrap-sass> has a number of major
